@@ -157,6 +157,11 @@ elsif (   ($define{USE_LOCALE_THREADS} || $define{USE_THREAD_SAFE_LOCALE})
     $define{USE_POSIX_2008_LOCALE} = 1 if $define{HAS_POSIX_2008_LOCALE};
 }
 
+if (   ($define{USE_POSIX_2008_LOCALE} && ! $define{HAS_QUERYLOCALE}))
+{
+    $define{USE_PL_CURLOCALES} = 1;
+}
+
 if (   $ARGS{PLATFORM} eq 'win32'
     && $define{USE_THREAD_SAFE_LOCALE}
     && $cctype < 140)
@@ -273,7 +278,7 @@ if ($ARGS{PLATFORM} ne 'vms') {
 
 if ($ARGS{PLATFORM} ne 'win32') {
     ++$skip{$_} foreach qw(
-		    Perl_my_setlocale
+		    Perl_get_win32_message_utf8ness
 			 );
 }
 
@@ -373,8 +378,7 @@ unless ($define{'USE_ITHREADS'}) {
 		    PL_env_mutex
 		    PL_hints_mutex
 		    PL_locale_mutex
-		    PL_lc_numeric_mutex
-		    PL_lc_numeric_mutex_depth
+		    PL_locale_mutex_depth
 		    PL_my_ctx_mutex
 		    PL_perlio_mutex
 		    PL_stashpad
@@ -416,7 +420,7 @@ unless ($define{USE_POSIX_2008_LOCALE})
         PL_underlying_numeric_obj
     );
 }
-unless ($define{USE_POSIX_2008_LOCALE} && ! $define{HAS_QUERY_LOCALE})
+unless ($define{USE_PL_CURLOCALES})
 {
     ++$skip{$_} foreach qw(
         PL_curlocales
@@ -448,11 +452,6 @@ unless ($define{'MULTIPLICITY'}) {
 			 );
 }
 
-if ($define{USE_THREAD_SAFE_LOCALE}) {
-    ++$skip{PL_lc_numeric_mutex};
-    ++$skip{PL_lc_numeric_mutex_depth};
-}
-
 unless ($define{'USE_DTRACE'}) {
     ++$skip{$_} foreach qw(
                     Perl_dtrace_probe_call
@@ -549,11 +548,16 @@ unless ($define{USE_LOCALE_COLLATE}) {
 
 unless ($define{USE_LOCALE_NUMERIC}) {
     ++$skip{$_} foreach qw(
-		    PL_numeric_local
                     PL_underlying_numeric_obj
 			 );
 }
 
+unless ($define{USE_LOCALE_CTYPE}) {
+    ++$skip{$_} foreach qw(
+		    PL_ctype_name
+			 );
+}
+
 unless ($define{'USE_C_BACKTRACE'}) {
     ++$skip{Perl_get_c_backtrace_dump};
     ++$skip{Perl_dump_c_backtrace};
