@@ -304,12 +304,11 @@ Perl_sv_does_sv(pTHX_ SV *sv, SV *namesv, U32 flags)
 
     /* create a PV with value "isa", but with a special address
      * so that perl knows we're really doing "DOES" instead */
-    methodname = newSV_type(SVt_PV);
+    methodname = newSV_type_mortal(SVt_PV);
     SvLEN_set(methodname, 0);
     SvCUR_set(methodname, strlen(PL_isa_DOES));
     SvPVX(methodname) = (char *)PL_isa_DOES; /* discard 'const' qualifier */
     SvPOK_on(methodname);
-    sv_2mortal(methodname);
     call_sv(methodname, G_SCALAR | G_METHOD);
     SPAGAIN;
 
@@ -733,7 +732,7 @@ XS(XS_PerlIO_get_layers)
     {
         SV *	sv;
         GV *	gv;
-        IO *	io;
+        IO *	io = NULL;
         bool	input = TRUE;
         bool	details = FALSE;
 
@@ -776,12 +775,16 @@ XS(XS_PerlIO_get_layers)
         }
 
         sv = POPs;
-        gv = MAYBE_DEREF_GV(sv);
 
-        if (!gv && !SvROK(sv))
-            gv = gv_fetchsv_nomg(sv, 0, SVt_PVIO);
+        /* MAYBE_DEREF_GV will call get magic */
+        if ((gv = MAYBE_DEREF_GV(sv)))
+            io = GvIO(gv);
+        else if (SvROK(sv) && SvTYPE(SvRV(sv)) == SVt_PVIO)
+            io = (IO*)SvRV(sv);
+        else if (!SvROK(sv) && (gv = gv_fetchsv_nomg(sv, 0, SVt_PVIO)))
+            io = GvIO(gv);
 
-        if (gv && (io = GvIO(gv))) {
+        if (io) {
              AV* const av = PerlIO_get_layers(aTHX_ input ?
                                         IoIFP(io) : IoOFP(io));
              SSize_t i;
@@ -1126,7 +1129,7 @@ XS(XS_NamedCapture_TIEHASH)
                 flag = SvTRUE(mark[1]) ? RXapif_ALL : RXapif_ONE;
             mark += 2;
         }
-        ST(0) = sv_2mortal(newSV_type(SVt_IV));
+        ST(0) = newSV_type_mortal(SVt_IV);
         sv_setuv(newSVrv(ST(0), package), flag);
     }
     XSRETURN(1);
